#!/bin/sh
. /usr/lib/ci-tests/sysklogd/sysklogd_add_functions.sh

KLOGDVERSION=1.5.0
#This module was chosen only for test purpose, because it doesn't have any dep. 
MODULE_NAME=wl12xx_spi
FULL_MODULE_NAME=`find /lib/modules/ -name "$MODULE_NAME.ko"`

STOP_LOGGING_STR="kernel: Kernel logging (proc) stopped."
KERNEL_LOGGING_STR="kernel:*"
KLOGD_STARTED_STR="^klogd*.*started."
LOAD_SM_STR="kernel: Loaded *.* symbols from *.* modules."

# Additional function to start klogd
start_klogd()
{
	# Do this before tests
	echo "$TEST_NAME: In start_klogd ($*) "
	rm -f $TEMP_FILE
	if [ `pidof klogd` ] ; then 
		kill_klogd
	fi
	
	echo "$TEST_NAME: Starting klogd"
	assert_cmd "$which_klogd $1 $2 $3 $4"
	echo "$TEST_NAME: klogd was started succesfully"
}

# Additional function to stop klogd
kill_klogd()
{
	echo "$TEST_NAME: Killing klogd"
	assert_cmd "kill -KILL `pidof klogd`"
	delay 2
}

# Additional function to remove and insert test module
remove_insert_module() 
{	
	echo "$TEST_NAME: In remove_insert_module ($*) "
	write_string "START"
	assert_cmd "rmmod $MODULE_NAME"
	assert_cmd "insmod $FULL_MODULE_NAME"
	write_string "STOP"
}
#=======================================================
# 
# Test signals influences on syslogd
#=======================================================
#Test case 1,2,3,4
test_kill_sig()
{
	start_klogd
	Pid=`pidof klogd`

	assert_cmd "kill -$1 $Pid"
	delay 2

	assert_condition "`pidof klogd` -ne $Pid" "Signal isn't caught. Daemon is not running or keeps on working. FAIL" "klogd"

	echo "$TEST_NAME: Daemon work was terminated. OK"
	restore_logd 
}


#Test case #1
test_SIGHUP_sig()
{
	test_kill_sig "SIGHUP"
}

#Test case #2
test_SIGKILL_sig()
{
	test_kill_sig "SIGKILL"
}

#Test case #3
test_SIGTERM_sig()
{
	test_kill_sig "SIGTERM"
}

#Test case #4
test_SIGINT_sig()
{
	test_kill_sig "SIGINT"
}


# Test case 5-6,7 # SIGTSTP & SIGCONT
test_SIGTSTP_sig()
{
	start_klogd
	assert_cmd "kill -SIGTSTP `pidof klogd`"
	
	remove_insert_module
	assert_cmd get_block_data_from_syslog 

	assert_condition "`grep "$STOP_LOGGING_STR" $TMP_BLOCK_DATA -c` -gt 0 " "SIGTSTP isn't caught. FAIL" "klogd"

	echo "$TEST_NAME: Verifying that klogd doesn't write anything to syslog"

	remove_insert_module
	assert_cmd get_block_data_from_syslog 
	assert_condition "`grep "kernel:" $TMP_BLOCK_DATA -c` -le 0 " "SIGTSTP is caught, but nothing changes. FAIL"

	echo "$TEST_NAME: Sending SIGCONT signal"
	
	assert_cmd "kill -SIGCONT `pidof klogd`"
	remove_insert_module	
	assert_cmd get_block_data_from_syslog 
	assert_condition "`grep "kernel:" $TMP_BLOCK_DATA -c` -gt 0 "  "Error: SIGCONT isn't caught. FAIL"

	echo "$TEST_NAME: SIGCONT & SIGSTP work as expected. OK"
	restore_logd 
}
 
compare_loaded_symbols() # $1 - RUN KLOGD
{
	echo "$TEST_NAME: In compare_loaded_symbols ($*) "
	write_string "START"
	assert_cmd $1 
	assert_cmd "rmmod $MODULE_NAME"
	delay 10
	write_string "STOP"

	assert_cmd get_block_data_from_syslog	
	symbols_bl=`grep "$LOAD_SM_STR" $TMP_BLOCK_DATA | awk '{print $7}'`
	modules_bl=`grep "$LOAD_SM_STR" $TMP_BLOCK_DATA | awk '{print $10}'`
	
	echo "$TEST_NAME: Middle results: Loaded $symbols_bl symbols"
	echo "$TEST_NAME: Middle results: Loaded $modules_bl modules"
	delay 1

	write_string "START"
	assert_cmd $1
	assert_cmd "insmod $FULL_MODULE_NAME"
	delay 10
	write_string "STOP"

	assert_cmd get_block_data_from_syslog	
	symbols_al=`grep "$LOAD_SM_STR" $TMP_BLOCK_DATA | awk '{print $7}'`
	modules_al=`grep "$LOAD_SM_STR" $TMP_BLOCK_DATA | awk '{print $10}'`

	echo "$TEST_NAME: Final results: Loaded $symbols_al symbols"
	echo "$TEST_NAME: Final results: Loaded $modules_al modules"

	assert_condition "$modules_al -eq `expr $modules_bl + 1` " "SIGUSR1 isn't caught. FAIL"
	assert_condition "$symbols_al -gt $symbols_bl" "SIGUSR1 isn't caught. FAIL"
	
	echo "$TEST_NAME: Daemon klogd reloaded symbols. OK"
	restore_logd 		
}

# Test case #8 SIGUSR1
test_SIGUSR1_sig()
{ 
	start_klogd
	assert_cmd 'compare_loaded_symbols "kill -SIGUSR1 `pidof klogd`"'
	restore_logd "klogd"		
}

#=======================================================
# 
# Test launching syslogd with different options
#
#=======================================================
# Test case: option -f 
test_option_f()
{
	start_klogd -f $TEMP_FILE
	remove_insert_module
	assert_cmd get_block_data_from_syslog

	echo "$TEST_NAME: File $TEMP_FILE contains"
	cat  $TEMP_FILE

	assert_condition "-e $TEMP_FILE " 	"Option -f doesn't work. FAIL"
	assert_condition "`grep "$KLOGD_STARTED_STR" $TEMP_FILE -c` -gt 0 " "Option -f doesn't work. FAIL"
	assert_condition "`grep "$KERNEL_LOGGING_STR"  $TMP_BLOCK_DATA -c` -eq 0 " "Option -f doesn't work. FAIL"

	echo "$TEST_NAME: Kernel writes all messages in scpecified file. OK"
	restore_logd 		
}

# Test case: option -o 
test_option_o()
{
	start_klogd -o -f $TEMP_FILE "&"
	Pid=`pidof klogd`
	echo "$TEST_NAME: Pid = $Pid"
	remove_insert_module
	if [ `pidof klogd` ] ; then
		assert_condition "`pidof klogd` -ne $Pid" "Option -o doesn't work. Daemon klogd still work, but should exit"
	fi
	echo "$TEST_NAME: klogd -o has terminated after reading buffer. OK"
	restore_logd	
}

# Test case 
test_option_I()
{ 
	start_klogd
	assert_cmd 'compare_loaded_symbols "klogd -I"'
	restore_logd 			
}

#Test case: option -n
#Do not fork
test_option_n()
{
	start_klogd -n "&"

	Pid=`pidof klogd`
	PPid=`cat /proc/$Pid/status  | grep PPid: | awk '{print $2}'`
	echo "$TEST_NAME: Pid = $Pid, PPid= $PPid" 

	assert_condition "$PPid -ne 1 " "Daemon klogd makes forking and becomes a daemon. FAIL"

	echo "$TEST_NAME: Syslogd avoids forking. OK"
	restore_logd
}

#=======================================================
# 
# Start main programm
#
#=======================================================

print_usage ()
{
	echo "Usage: $0 -s SIG_NAME runs klogd test for input SIG_NAME;" 
	echo "       Possible signals {SIGUSR1, SIGHUP, SIGINT, SIGTERM, SIGCONT, SIGTSTP, SIGKILL} "
	echo " "
	echo "Usage: $0 -o option runs klogd test for input option."
	echo "       Possible options {f, o, n, I} "
	echo " "
	exit 1
}
SYSTEM="klogd"

while getopts "s:o:" opt; do
	case $opt in
		s )
			case $OPTARG in 
				"SIGHUP" | "SIGTERM" | "SIGUSR1" | "SIGINT" | "SIGKILL" | "SIGTSTP" | "SIGCONT" ) 
					if  [ "$OPTARG" = "SIGCONT" ] ; then
						OPTARG="SIGTSTP"
					fi
					TEST_NAME="test_"$OPTARG"_sig" 	
					print_test_header  
					assert_cmd $TEST_NAME 
					print_test_results 0 ;;
				*) 
					echo "Unknown parameter value..."
					print_usage
			esac;;
						
        o )
			case $OPTARG in
				"v" | "f" | "o" | "I" | "n" )
					TEST_NAME="test_option_$OPTARG" 
					print_test_header  
					assert_cmd $TEST_NAME 
					print_test_results 0 ;;
				*)
					echo "Unknown parameter value..."
					print_usage
			esac;;
        * )echo $USAGE
		exit 1;;
   esac
done

